use core::panic;
use std::{
    error::Error,
    ops::{Div, DivAssign, Rem, RemAssign},
};

use super::{Number, Sign};

impl Number {
    pub fn div_with_rem(n1: Number, n2: Number) -> Result<(Number, Number), Box<dyn Error>> {
        if n2 == 0.into() {
            return Err(Box::from("Division by 0"));
        }
        let n1_len = n1.digits.len();
        let n2_len = n2.digits.len();
        if n2_len > n1_len {
            return Ok((Number::from(0), n2));
        }
        let dividend = n1.digits[..n2_len].to_vec();
        let mut quotient = vec![];
        let mut remainder = Number {
            digits: dividend.clone(),
            sign: Sign::Positif,
        };
        let mut iteration = 1;
        loop {
            let mut factor = 0;
            loop {
                let temp_remainder = remainder.clone() - n2.clone();
                if temp_remainder.sign == Sign::Negatif {
                    quotient.push(factor);
                    break;
                }
                remainder = temp_remainder;
                factor += 1;
            }
            if n1_len == n2_len + iteration - 1 {
                break;
            }
            remainder.digits.push(n1.digits[n2_len + iteration - 1]);
            iteration += 1;
        }
        let mut res = Number {
            digits: quotient,
            sign: Sign::Positif,
        };
        for digit in res.clone().digits {
            if digit != 0 || res.digits.len() == 1 {
                break;
            }
            res.digits.remove(0);
        }
        for digit in remainder.clone().digits {
            if digit != 0 || remainder.digits.len() == 1 {
                break;
            }
            remainder.digits.remove(0);
        }
        Ok((res, remainder))
    }

    pub fn gcd(mut a: Number, mut b: Number) -> Number {
        loop {
            let t = b.clone();
            b = a % b;
            a = t;
            if b == Number::from(0) {
                return a;
            }
        }
    }
}

impl Div for Number {
    type Output = Self;

    fn div(self, rhs: Self) -> Self::Output {
        Self::div_with_rem(self, rhs).unwrap().0
    }
}

impl DivAssign for Number {
    fn div_assign(&mut self, rhs: Self) {
        match Self::div_with_rem(self.clone(), rhs) {
            Ok((new, _)) => {
                self.digits = new.digits;
                self.sign = new.sign;
            }
            Err(e) => panic!("{e}"),
        }
    }
}

impl Rem for Number {
    type Output = Self;

    fn rem(self, rhs: Self) -> Self::Output {
        Self::div_with_rem(self, rhs).unwrap().1
    }
}

impl RemAssign for Number {
    fn rem_assign(&mut self, rhs: Self) {
        match Self::div_with_rem(self.clone(), rhs) {
            Ok((_, new)) => {
                self.digits = new.digits;
                self.sign = new.sign;
            }
            Err(e) => panic!("{e}"),
        }
    }
}
